//
//  CCPsrController.m
//  Pauser
//
//  Created by 2510 on Mon Oct 14 2002.
//  Copyright (c) 2002 by 2510. All rights reserved.
//

#import "CCPsrController.h"
#import "CCMenuItemBridge.h"

#import <mach/mach.h>

static NSString	*pidListKeyPID = @"pid";
static NSString *pidListBridgeObject = @"bridge";

@implementation CCPsrController
- (id)init
{
	[super init];
	suspendList = [[NSMutableDictionary alloc] initWithCapacity:0];
	return self;
}

- (NSArray *)allProcesses
{
int			my_pid;
int			last_pid;
int			processes = 0;

	if(pidList != nil) [pidList release];
	if(pidToName != nil) [pidToName release];
	pidList = [[NSMutableArray alloc] initWithCapacity:0];
	pidToName = [[NSMutableDictionary alloc] initWithCapacity:0];
	
	// PID -> name bNAbv\z
	{
	NSData	*data;
	
		{
		NSTask	*task;
		NSPipe	*pipe;
		
			task = [[NSTask alloc] init];
			[task setLaunchPath:@"/bin/ps"];
			[task setArguments:[NSArray arrayWithObjects:@"-xwww",@"-o pid,command",nil]];
			pipe = [NSPipe pipe];
			[task setStandardOutput:pipe];
			[task launch];
			last_pid = [task processIdentifier];
			[task waitUntilExit];
			data = [[pipe fileHandleForReading] availableData];
		}
		
		{
		int			length;
		int			pos;
		const char	*buffer = [data bytes];
		const char	*line_start = NULL;
		int			line_length = 0;
		
			length = [data length];
			for(pos = 0;pos < length;pos ++)
				if(buffer[pos] == 0x00)
					break;
				else if(buffer[pos] == 0x0A)
				{
				int		offset;
				
					if(line_start)
					{
					int			pid;
					NSString	*applicationName = nil;
					
						pid = [[NSString stringWithCString:line_start length:7] intValue];
						
						{
						int		slash = 0;
						
							for(offset = 0;offset < line_length;offset ++)
							{
								if(line_start[offset] == '.')
									if(strncmp(&line_start[offset],".app",4) == 0)
									{
										applicationName = [NSString stringWithCString:&line_start[slash] length:offset - slash];
										break;
									} else
										offset += 3;
								else if(line_start[offset] == '/')
									slash = offset + 1;
							}
						}
						
						if(applicationName)
						{
							[pidToName setObject:applicationName forKey:[NSString stringWithFormat:@"%d",pid]];
						
						//	NSLog([NSString stringWithFormat:@"%d as %@",pid,applicationName]);
						}
					}
					line_start = buffer + pos + 1;
					line_length = 0;
				} else
					line_length ++;
		}
	}
	
	my_pid = [[NSProcessInfo processInfo] processIdentifier];
	
	{
	mach_port_t	task_self = mach_task_self();
	mach_port_t	temp;
	int			pid;
	
		for(pid = 1;pid < last_pid;pid ++)
			if(pid != my_pid)
				if(task_for_pid(task_self,pid,&temp) == KERN_SUCCESS)
				{
				NSMutableDictionary	*dictionary;
				CCMenuItemBridge	*bridge;
				
					dictionary = [[NSMutableDictionary alloc] initWithCapacity:0];
					[dictionary setObject:[NSString stringWithFormat:@"%d",pid] forKey:pidListKeyPID];
					
					bridge = [[[CCMenuItemBridge alloc] init] autorelease];
					[bridge setTarget:self];
					[bridge setAction:@selector(toggle:)];
					[bridge setContext:dictionary];
					[dictionary setObject:bridge forKey:pidListBridgeObject];
					
					[pidList addObject:dictionary];
					processes ++;
					if(processes >= 64) break;
				}
	}
	
	return pidList;
}

- (BOOL)suspendTask:(int)pid
{
mach_port_t	task;

	if(task_for_pid(mach_task_self(),pid,&task) == KERN_SUCCESS)
	{
	//	NSLog([NSString stringWithFormat:@"suspend task %d",pid]);
		task_suspend(task);
		return YES;
	} else
	{
		NSLog([NSString stringWithFormat:@"failed to suspend task %d",pid]);
		return NO;
	}
}

- (BOOL)resumeTask:(int)pid
{
mach_port_t	task;

	if(task_for_pid(mach_task_self(),pid,&task) == KERN_SUCCESS)
	{
	//	NSLog([NSString stringWithFormat:@"resume task %d",pid]);
		task_resume(task);
		return YES;
	} else
	{
		NSLog([NSString stringWithFormat:@"failed to resume task %d",pid]);
		return NO;
	}
}

- (void)toggle:(id)sender
{
id			object;
NSString	*pidString;
int			pid;

	object = [sender context];
	pidString = [object objectForKey:pidListKeyPID];
	pid = [[object objectForKey:pidListKeyPID] intValue];
	
	if([suspendList objectForKey:pidString])
	{
		[suspendList removeObjectForKey:pidString];
		[self resumeTask:pid];
	} else
	{
		[suspendList setObject:object forKey:pidString];
		[self suspendTask:pid];
	}
}

- (NSMenu *)applicationDockMenu:(NSApplication *)sender
{
	[self allProcesses];
	
	if(processMenu) [processMenu release];
	processMenu = [[NSMenu allocWithZone:[NSMenu menuZone]] initWithTitle:@""];
	
	{
	NSEnumerator	*enumerator;
	id				object;
	
		enumerator = [pidList objectEnumerator];
		while((object = [enumerator nextObject]) != nil)
		{
		NSMenuItem	*item;
		NSString	*pidString,
					*applicationName;
		NSString	*title;
		
			pidString = [object objectForKey:pidListKeyPID];
			applicationName = [pidToName objectForKey:pidString];
			if(applicationName)
				title = applicationName;
			else
				title = [NSString stringWithFormat:@"(Process %d)",[pidString intValue]];
			title = [([suspendList objectForKey:pidString] ? @"Resume " : @"Suspend ") stringByAppendingString:title];
			item = [[NSMenuItem alloc] initWithTitle:title action:@selector(perform:) keyEquivalent:@""];
			[item setTarget:[object objectForKey:pidListBridgeObject]];
			[processMenu addItem:item];
		}
	}
	return processMenu;
}

- (void)applicationWillTerminate:(NSNotification *)notification
{
NSEnumerator	*enumerator;
id				object;

//	NSLog(@"terminate");
	enumerator = [[suspendList allValues] objectEnumerator];
	while((object = [enumerator nextObject]) != nil)
	{
	int		pid;
	
		pid = [[object objectForKey:pidListKeyPID] intValue];
		[self resumeTask:pid];
	}
}
@end
